#!/usr/bin/php
<?php
require_once '/var/www/init.php';

use Console\DataStorageSyncBackups;
use Console\DataStorageSyncRecordings;
use Logger\Log;
use Wildix\Core\Container;
use Wildix\Settings\Env\EnvConfig;

const FILE_LOCK = '/tmp/sync_data.lock';

function printLog($message, $error = false) {
    $log = Container::get(Log::class);
    dump($message);

    if ($error) {
        $log->error(__LINE__, $message);
    } else {
        $log->info(__LINE__, $message);
    }
}

function getLastUploadedRecordTimestamp() {
    return (int)file_get_contents(DataStorageSyncRecordings::LAST_SYNCED_RECORD_FILE) ?? 0;
}

function syncData() {
    $envConfig = Container::get(EnvConfig::class);
    $serial = Container::get(\Models\PBX::class)->getSerial();

    printLog('Uploading call recordings...');
    $syncCallRecordings = new DataStorageSyncRecordings(
        [
            'createdFrom' => getLastUploadedRecordTimestamp(),
            'saveTimestamp' => true
        ],
        Container::getInstance()
    );

    $syncCallRecordings->run();
    printLog('Done');

    printLog('Uploading backups...');
    $syncBackups = new DataStorageSyncBackups([], Container::getInstance());
    $syncBackups->run();
    printLog('Done');

    if (count($syncCallRecordings->getItems(getLastUploadedRecordTimestamp())) === 0
        && count($syncBackups->getItems()) === 0
    ) {
        printLog('Setting up cloud data finished. Enabling Cloud Data Storage...');
        $envConfig->set('SERVICE_DATA_STORAGE_ENABLE', true);
        $envConfig->save();
        exec('sudo systemctl stop sync-data-to-data-storage.timer && sudo systemctl disable sync-data-to-data-storage.timer &>/dev/null');
        printLog('WMS-interface Cloud Data Storage is enabled ' . $serial);
    } else {
        printLog('It seems new data appeared. Re-uploading');
        syncData();
    }
}

$handle = fopen(FILE_LOCK, 'w');
if (!flock($handle, LOCK_EX | LOCK_NB)) {
    printLog('Data sync is already running');
    die();
}

if (env('SERVICE_DATA_STORAGE_ENABLE', false)) {
    printLog('Cloud Data Storage is already enabled');
    die();
}

try {
    syncData();
} catch (Throwable $e) {
    $msg = [
        'msg' => 'Error occurred while uploading files to Cloud Data Storage',
        'serial' => Container::get(\Models\PBX::class)->getSerial(),
        'error' => $e->getMessage(),
        'stacktrace' => $e->getTraceAsString(),
    ];
    printLog($msg, true);
    exit(1);
} finally {
    fclose($handle);
    unlink(FILE_LOCK);
}
